# -*- coding: utf-8 -*-

''' Componentes de la interfaz de usuario. '''

import threading
import thread
import time
from random             import randint
from os                 import sep
from datetime           import date
from copy               import deepcopy

import pygtk
pygtk.require ('2.0')
import gtk
import gobject

from Pyro.errors        import ConnectionClosedError, ProtocolError

from common.config      import PROVINCES, CONNECTED_ICON_FILE, BIG_VALUE, \
                               WARNING_ICON_FILE, CONFIG_DIR, LOG_TAIL_LENGTH
from common.gui.status  import show_status, put_title_in_label
from common.utils       import ui_connection_safe

class Centres:

    ''' Listado de centros en forma de árbol. '''

    def __init__ (self, tree, info, province = None):

        ''' Crear un listado de centros Guadalinfo.

            @param tree: control I{árbol} al que asociar el listado.
            @type tree: L{gtk.Treeview}
            @param info: información sobre los centros (conexión, incidencias, etc.).
            @type info: L{dict} de L{dict} etc.
            @param province: código de provincia (si el GUI es de tipo I{diputación}).
            @type province: L{int} o L{None} '''

        # Se fijan los atributos de instancia:
        self.tree = tree
        self.info = info
        self.province = province

        # Diccionario con el número de incidencias de cada centro. Se usa para
        # detectar cambios en las incidencias del centro
        self.old_ninc = {}

        # Se crea la estructura del árbol:
        self.create_structure ()

        # Se puebla el árbol por primera vez:
        if self.info != {}:
            self.populate ()
        

    def create_structure (self):

        ''' Inicializar la estructura del árbol de centros. '''

        self.model = gtk.TreeStore (str,            #  0 Código de provincia          (no se muestra)
                                    str,            #  1 Código de municipio          (no se muestra)
                                    str,            #  2 Nombre normalizado (plano)   (no se muestra)
                                    str,            #  3 Si está conectado            (no se muestra)
                                    str,            #  4 Nº de días desde últ. conex. (no se muestra)
                                    str,            #  5 Si tiene incidencias         (no se muestra)
                                    str,            #  6 Nº de incidencias            (no se muestra)
                                    str,            #  7 Nombre del centro
                                    gtk.gdk.Pixbuf, #  8 Icono de conexión
                                    str,            #  9 Nº de días desde últ. conex.
                                    gtk.gdk.Pixbuf, # 10 Icono de aviso
                                    str,            # 11 Nº de incidencias
                                    str)            # 12 Comentario
        self.provNodes = {}
        self.rowStatus = {}

        for i in PROVINCES:
            self.rowStatus [str (i [0])] = True

        self.connected = gtk.gdk.pixbuf_new_from_file (CONNECTED_ICON_FILE)
        self.warning = gtk.gdk.pixbuf_new_from_file (WARNING_ICON_FILE)
        self.onlyConnected = False
        self.onlyProblematic = False

#         self.markedPath = None
        self.iter = None

        # Distintos CellRender:
        rendererText = gtk.CellRendererText ()
        rendererIcon = gtk.CellRendererPixbuf ()

        # Columnas del árbol:

        # codPro:
        # (usaremos después esta columna "dummy" para ocultar el "expander")
        dummyColumn = gtk.TreeViewColumn (None, rendererText, text = 0)
        dummyColumn.set_sort_column_id (0)
        dummyColumn.set_visible (False)
        self.tree.append_column (dummyColumn)

        # codMun:
        tmp = gtk.TreeViewColumn (None, rendererText, text = 1)
        tmp.set_sort_column_id (1)
        tmp.set_visible (False)
        self.tree.append_column (tmp)

        # Nombre normalizado del centro:
        rightNameColumn = gtk.TreeViewColumn (None, rendererText, text = 2)
        rightNameColumn.set_sort_column_id (2)
        rightNameColumn.set_visible (False)
        self.tree.append_column (rightNameColumn)

        # Connected:
        tmp = gtk.TreeViewColumn (None, rendererText, text = 3)
        tmp.set_sort_column_id (3)
        tmp.set_visible (False)
        self.tree.append_column (tmp)

        # Days:
        tmp = gtk.TreeViewColumn (None, rendererText, text = 4)
        tmp.set_sort_column_id (4)
        tmp.set_visible (False)
        self.tree.append_column (tmp)

        # ¿Hay incidencias?
        tmp = gtk.TreeViewColumn (None, rendererText, text = 5)
        tmp.set_sort_column_id (5)
        tmp.set_visible (False)
        self.tree.append_column (tmp)

        # Número de incidencias:
        tmp = gtk.TreeViewColumn (None, rendererText, text = 6)
        tmp.set_sort_column_id (6)
        tmp.set_visible (False)
        self.tree.append_column (tmp)

        # Nombre del centro:
        columnCentre = gtk.TreeViewColumn ('Centro', rendererText, markup = 7)
        columnCentre.set_clickable (True)
        columnCentre.set_reorderable (False)
        columnCentre.set_sort_column_id (2)
        columnCentre.set_resizable (True)
        self.tree.append_column (columnCentre)

        # Conexión:
        columnConn = gtk.TreeViewColumn (None, rendererIcon, pixbuf = 8)
        columnConn.set_clickable (True)
        columnConn.set_reorderable (False)
        columnConn.set_sort_column_id (3)
        columnConn.set_resizable (False)
        columnConn.set_sizing (gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree.append_column (columnConn)

        # Nº de días desde última conexión:
        columnConn2 = gtk.TreeViewColumn ("Días", rendererText, markup = 9)
        columnConn2.set_clickable (True)
        columnConn2.set_reorderable (False)
        columnConn2.set_sort_column_id (4)
        columnConn2.set_resizable (False)
        columnConn2.set_sizing (gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree.append_column (columnConn2)

        # Aviso:
        columnWarn = gtk.TreeViewColumn (None, rendererIcon, pixbuf = 10)
        columnWarn.set_clickable (True)
        columnWarn.set_reorderable (False)
        columnWarn.set_sort_column_id (5)
        columnWarn.set_resizable (False)
        columnWarn.set_sizing (gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree.append_column (columnWarn)

        # Incidencias:
        columnWarn2 = gtk.TreeViewColumn ("Nº inc.", rendererText, markup = 11)
        columnWarn2.set_clickable (True)
        columnWarn2.set_reorderable (False)
        columnWarn2.set_sort_column_id (6)
        columnWarn2.set_resizable (False)
        columnWarn2.set_sizing (gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.tree.append_column (columnWarn2)

        # Comentarios:
        columnComm = gtk.TreeViewColumn ('Comentarios', rendererText, markup = 12)
        columnComm.set_clickable (True)
        columnComm.set_reorderable (False)
        columnComm.set_sort_column_id (12)
        columnComm.set_resizable (True)
        self.tree.append_column (columnComm)

        # Se establecen diversas propiedades del árbol:
        self.tree.set_headers_clickable (True)
        self.tree.set_reorderable (False)
        self.tree.set_enable_search(True)
        self.tree.set_search_column(7)

        if self.province is None:
            self.tree.set_expander_column (columnCentre)
        else:
            self.tree.set_expander_column (dummyColumn)

        self.tree.set_rules_hint (True)
        self.defaultSort ()
        self.tree.set_model (self.model)

        # Un par de manejadores que solo tienen que ver con el árbol de centros:
        self.tree.connect ('row_collapsed', self.rowCollapsedOrExpanded)
        self.tree.connect ('row_expanded', self.rowCollapsedOrExpanded)

    def populate (self, selected = None, GUI = None):

        ''' Poblar el árbol de centros.

            @param selected: código del centro seleccionado, si es que hay alguno.
            @type selected: L{int} o L{None}
            @param GUI: instancia de la GUI que se desea poblar.
            @type GUI: L{common.gui.gui.GraphicalUserInterface} '''

        if GUI:
            self.info = GUI.info

        self.unmark ()

        # Se borra el modelo asociado al árbol de centros:

        if self.tree.get_model () is not None:
            self.tree.get_model ().clear ()

        # Si el tipo del GUI es «servidor» (es decir, si se muestran los centros
        # de todas las provincias) entonces se crean las 8 provincias como ramas
        # principales del árbol:

        if self.province is None:

            for i in PROVINCES:
                iter = self.model.insert (None, 0)
                self.model.set_value(iter, 0, -1)
                self.model.set_value(iter, 1, -1)
                self.model.set_value(iter, 2, i [1])
                self.model.set_value(iter, 7, i [1])
                self.provNodes [str (i [0])] = iter

        # Para cada centro, se añade su fila correspondiente al árbol
        selected_iter = None

        # Lista de códigos de centros que han cambiado el número de incidencias
        ninc_changes = []
        for i in self.info:

            for j in self.info [i]:
                tmp = self.info [i] [j]

                # Se comprueba si el centro «tmp» pasa todos los filtros:

                if (not self.onlyConnected or tmp ['conn']) and \
                       (not self.onlyProblematic or tmp ['alert']):

                    if self.province is None:
                        iter = self.model.insert (self.provNodes [str (tmp ['provCode'])], 0)
                    else:
                        iter = self.model.insert (None, 0)

                    self.model.set_value (iter, 0, str (i))
                    self.model.set_value (iter, 1, str (j))

                    self.model.set_value (iter, 2, tmp ['munName'])
                    self.model.set_value (iter, 7, tmp ['munName'])

                    # Las columnas ocultas que indican nº de días y nº de incidencias
                    # (4ª y 6ª) almacenan cadenas, así que para ordenar números
                    # correctamente, le sumamos siempre una cantidad grande (BIG_VALUE).
                    # Así al pasarlos a cadenas el orden lexicográfico coincide con
                    # el matemático. Esta cantidad debe ser tal que incluso al
                    # sumarle -1 (para indicar error o falta de datos) se mantenga
                    # el orden lexicográfico correcto (no serviría «100000», por ejemplo).

                    if tmp ['conn']:
                        self.model.set_value (iter, 3, '1')
                        self.model.set_value (iter, 4, BIG_VALUE - 1)
                        self.model.set_value (iter, 8, self.connected)
                    else:

                        if tmp ['last'] is not None:
                            days = date.today ().toordinal () - int (tmp ['last'])
                            self.model.set_value (iter, 9, days)
                        else:
                            days = BIG_VALUE
                            self.model.set_value (iter, 9, '?')

                        self.model.set_value (iter, 3, '0')
                        self.model.set_value (iter, 4, BIG_VALUE + days)

                    centre_code = (tmp['provCode'], tmp['munCode'] )
                    if tmp ['alert']:

                        if tmp ['warn'] is not None:
                            no_of_inc = int (tmp ['warn'])
                            self.model.set_value (iter, 11, no_of_inc)
                        else:
                            no_of_inc = BIG_VALUE
                            self.model.set_value (iter, 11, '?')


                        self.model.set_value (iter, 5, '1')
                        self.model.set_value (iter, 6, BIG_VALUE + no_of_inc)
                        self.model.set_value (iter, 10, self.warning)
                    else:
                        self.model.set_value (iter, 5, '0')
                        self.model.set_value (iter, 6, BIG_VALUE)
                        no_of_inc = 0

                    if self.old_ninc.has_key(centre_code):
                        if self.old_ninc[centre_code] != no_of_inc:
                            ninc_changes.append(centre_code)
                    else:
                        ninc_changes.append(centre_code)

                    self.old_ninc[centre_code] = no_of_inc

                    if tmp ['comm'] is not None:
                        self.model.set_value (iter, 12, tmp ['comm'])

                    if centre_code == selected:
                        selected_iter = iter

        previousStatus = deepcopy (self.rowStatus)

        for i in previousStatus.keys ():

            if previousStatus [i]:
                self.tree.expand_row (self.tree.get_model ().get_path (self.provNodes [i]), False)
            else:
                self.tree.collapse_row (self.tree.get_model ().get_path (self.provNodes [i]))

        if selected_iter:
            selection =  self.tree.get_selection()
            selection.select_iter(selected_iter)
            path = self.model.get_path(selected_iter)
            if selected in ninc_changes:
                GUI.rowActivated(self.tree, path, False)
            self.mark(path)

    def rowCollapsedOrExpanded (self, tree, iter, path):

        ''' Repasar el árbol para comprobar el estado de todas las ramas.
            Se comprueba, para todas las provincias, si están estiradas o contraídas.

            @param tree: control I{árbol} asociado al listado.
            @type tree: L{gtk.Treeview}
            @param iter: sin utilidad.
            @type iter: irrelevante
            @param path: sin utilidad.
            @type path: irrelevante '''

        for i in self.provNodes.keys ():
            self.rowStatus [i] = self.tree.row_expanded (self.tree.get_model ().get_path (self.provNodes [i]))

    def showOnlyConnected (self, value):

        ''' Excluir (o no) del listado de centros aquellos que no estén conectados.

            @param value: indica si mostrar B{solo} los centros activos.
            @type value: L{bool} '''

        self.onlyConnected = bool (value)
        self.unmark ()
        self.populate ()
#        self.checkModel ()

    def showOnlyProblematic (self, value):

        ''' Excluir (o no) del listado de centros aquellos que no tengan incidencias.

            @param value: indica si mostrar B{solo} los centros con incidencias activas.
            @type value: L{bool} '''

        self.onlyProblematic = bool (value)
        self.unmark ()
        self.populate ()
#        self.checkModel ()

    def defaultSort (self):

        ''' Ordenar el árbol de centros según el criterio de ordenación por defecto. '''

        # Los órdenes son "acumulables"; por eso indicamos varias columnas.
        # En caso de "empate" en la columna más "fuerte" (la última), se
        # resuelve por el valor de la anterior, y así sucesivamente.
        self.model.set_sort_column_id (6, gtk.SORT_ASCENDING) # Nº de incid.
        self.model.set_sort_column_id (5, gtk.SORT_ASCENDING) # ¿Incid.?
        self.model.set_sort_column_id (4, gtk.SORT_ASCENDING) # Nº de días
        self.model.set_sort_column_id (3, gtk.SORT_ASCENDING) # ¿Conectado?
        self.model.set_sort_column_id (2, gtk.SORT_ASCENDING) # Nombre centro.

    def checkModel (self):

        ''' Cambiar la propiedad de I{«clicabilidad»} :¬P de las cabeceras del árbol.
            Si hay algún filtro activo (I{solo conectados} y/o I{solo problemáticos})
            entonces las cabeceras de las columnas del listado B{no} serán sensibles
            al I{clic} del ratón. En otro caso, sí serán I{clicables}.
            B{Actualmente no se usa}. '''

        pass

#         self.tree.expand_all ()

#         if self.onlyConnected or self.onlyProblematic:
#             self.tree.set_model (self.filterModel)
#             self.filterModel.refilter ()

#             for i in self.tree.get_columns ():
#                 i.set_clickable (False)

#         else:
#             self.tree.set_model (self.model)

#             for i in self.tree.get_columns ():
#                 i.set_clickable (True)

#         self.defaultSort ()

    def collapse (self):

        ''' I{Contraer} todas las ramas (provincias) del árbol. '''

        self.tree.collapse_all ()

    def expand (self):

        ''' I{Estirar} todas las ramas (provincias) del árbol. '''

        self.tree.expand_all ()

    def mark (self, path):

        ''' Marcar una fila del listado, poniendo los textos en negrita.

            @param path: ruta (nodo) del árbol que hay que poner en negrita.
            @type path: L{gtk.Path} '''

#         self.markedPath = path
#         self.cursor = self.tree.set_cursor (path)

        model = self.tree.get_model ()
        self.iter = model.get_iter (path)

        # Columnas del árbol de centros que contienen texto
        # y por tanto pueden ser "marcadas" en negrita:
        markable_columns = (7, 9, 11, 12)

        for i in markable_columns:

            if model.get_value (self.iter, i) is not None:
                model.set_value (self.iter, i, '<b>' + str (model.get_value (self.iter, i)) + '</b>')

    def unmark (self):

        ''' I{Desmarcar} la fila activa del listado, quitando la negrita de todos sus
            textos. '''

#        if self.markedPath is not None:

        if self.iter is not None:
            model = self.tree.get_model ()

            # Columnas del árbol de centros que contienen texto
            # y por tanto pueden ser "marcadas" en negrita:
            markable_columns = (7, 9, 11, 12)

            for i in markable_columns:

                if model and (model.get_value (self.iter, i) is not None):
                    model.set_value (self.iter, i, str (model.get_value (self.iter, i)) [3:-4])

            self.iter = None

#             self.markedPath = None
#             self.cursor = None

class ExtendedInfo:

    ''' Elemento genérico de información detallada, asociada a un centro Guadalinfo
        concreto. '''
    
    def __init__ (self, where, visibilityCondition, server_ip, currentCentre = None):

        ''' Crear un I{control de información} genérico.

            @param where: control que mostrará la información.
            @type where: L{gtk.Widget}
            @param visibilityCondition: control cuya visibilidad condicionará la propia
                                        visibilidad de la información.
                                        Habitualmente será el I{widget} I{padre} de
                                        L{where}.
            @type visibilityCondition: L{gtk.Widget}
            @param server_ip: dirección IP del servidor.
            @type server_ip: L{str}
            @param currentCentre: centro Guadalinfo seleccionado inicialmente.
            @type currentCentre: no definido '''

        self.where = where
        self.visibility = visibilityCondition
        self.server_ip = server_ip
        self.centre = currentCentre
        self.update ()

    def change (self, centre, force = False):

        ''' Cambiar el centro Guadalinfo seleccionado.

            @param centre: nuevo centro Guadalinfo.
            @type centre: no definido
            @param force: indica si debe debe forzarse la selección de L{centre} incluso
                          si coincide con el centro que estaba previamente seleccionado.
            @type force: L{bool} '''

        if centre != self.centre or force:
            self.centre = centre
            self.update ()

    def getSelected (self):

        ''' Obtener el centro actualmente seleccionado.

            @return: el centro seleccionado.
            @rtype: no definido '''

        return self.centre

    def update (self):

        ''' Actualizar el control de información.
            Si el control que determina la visibilidad está visible B{y} hay algún
            centro Guadalinfo seleccionado, entonces se habilita el control y se
            hace visible. En otro caso, el control asociado a este objeto se
            deshabilita. '''

        if self.centre is not None and self.visibility.get_property ('visible'):

            if not self.where.get_property ('sensitive'):
                self.where.set_sensitive (True)

        else:

            if self.where.get_property ('sensitive'):
                self.where.set_sensitive (False)

class Status (ExtendedInfo):

    ''' Elemento de información detallada sobre el estado de un centro.
        Se corresponde con el I{panel de estado} del GUI de I{Vaguada}. '''

    def __init__ (self, where, visibilityCondition, server_ip, label, currentCentre = None):

        ''' Crear un objeto asociado al panel de estado.

            @param where: control que mostrará la información.
            @type where: L{gtk.Widget}
            @param visibilityCondition: control cuya visibilidad condicionará la propia
                                        visibilidad de la información.
                                        Habitualmente será el I{widget} I{padre} de
                                        L{where}.
            @type visibilityCondition: L{gtk.Widget}
            @param server_ip: dirección IP del servidor.
            @type server_ip: L{str}
            @param label: etiqueta o título del control asociado.
            @type label: L{gtk.Label}
            @param currentCentre: centro Guadalinfo seleccionado inicialmente.
            @type currentCentre: no definido '''

        self.label = label
        ExtendedInfo.__init__ (self, where, visibilityCondition, server_ip, currentCentre)

    def update (self):

        ''' Actualizar el panel de estado.
            Si el control que determina la visibilidad está visible B{y} hay algún
            centro Guadalinfo seleccionado, entonces se habilita el control y se
            hace visible. En otro caso, el control asociado a este objeto se
            deshabilita. '''

        ExtendedInfo.update (self)
        put_title_in_label (self.label)

        if self.centre is not None and self.visibility.get_property ('visible'):
            show_status (self.centre, self.where, self.label, self.server_ip)

class Log (ExtendedInfo):

    ''' Elemento de información sobre registros de actividad de un centro. '''

    def update (self):

        ''' Actualizar el registro.
            Si el control que determina la visibilidad está visible B{y} hay algún
            centro Guadalinfo seleccionado, entonces se habilita el control y se
            hace visible. En otro caso, el control asociado a este objeto se
            deshabilita. '''

        ExtendedInfo.update (self)

        if self.centre is not None and self.visibility.get_property ('visible'):
            self.__show ()

    def __show (self):

        ''' Redibujar el registro.
            Actualiza la presentación del registro en el GUI. '''

        if self.centre is not None and self.visibility.get_property ('visible'):

            try:
                uia = GraphicalUserInterface.PyroFactory.get_ui_assistant (self.server_ip)

                if 'Incidence' == self.type:
                    text = uia.get_inc_log (self.centre, LOG_TAIL_LENGTH)
                else:
                    text = uia.get_sync_log (self.centre, LOG_TAIL_LENGTH)

            except (ConnectionClosedError, ProtocolError):
                text = '[Se produjo un error al obtener el registro del centro]'

                if self.where.get_property ('sensitive'):
                    self.where.set_sensitive (False)

        else:
            text = ''

        if text == None:
            text = ''

        self.where.get_buffer ().set_text (text)

        while gtk.events_pending ():
            gtk.main_iteration ()

        buffer = self.where.get_buffer ()
        mark = buffer.get_mark ('endMark')
        self.where.scroll_to_mark (mark, 0.3, True, 0.2, 0.8)

#         container = self.where.get_parent ()
#         horizontalAdjustment = container.get_hscrollbar ().get_adjustment ()
#         verticalAdjustment = container.get_vscrollbar ().get_adjustment ()
#         maxScroll = verticalAdjustment.upper - verticalAdjustment.page_size
#         print self.__class__.__name__ + ' ' + str (verticalAdjustment.upper), \
#               str (verticalAdjustment.page_size), ' maxScroll: ' + str (maxScroll)
#         begin = self.where.get_buffer ().get_start_iter ()
#         end = self.where.get_buffer ().get_end_iter ()
#         horizontalAdjustment.set_value (0.0)
#         verticalAdjustment.set_value (maxScroll)

    __show == ui_connection_safe(__show)

class IncidenceLog (Log):

    ''' Información detallada sobre un registro de incidencias. '''

    def __init__ (self, where, visibilityCondition, currentCentre = None):

        ''' Crear un objeto asociado al I{registro de incidencias}.

            @param where: control que mostrará la información.
            @type where: L{gtk.Widget}
            @param visibilityCondition: control cuya visibilidad condicionará la propia
                                        visibilidad de la información.
                                        Habitualmente será el I{widget} I{padre} de
                                        L{where}.
            @type visibilityCondition: L{gtk.Widget}
            @param currentCentre: centro Guadalinfo seleccionado inicialmente.
            @type currentCentre: no definido '''

        Log.__init__ (self, where, visibilityCondition, currentCentre)
        self.type = 'Incidence'

class DispatchLog (Log):

    ''' Información detallada sobre un registro de envíos. '''

    def __init__ (self, where, visibilityCondition, currentCentre = None):

        ''' Crear un objeto asociado al I{registro de envíos}.

            @param where: control que mostrará la información.
            @type where: L{gtk.Widget}
            @param visibilityCondition: control cuya visibilidad condicionará la propia
                                        visibilidad de la información.
                                        Habitualmente será el I{widget} I{padre} de
                                        L{where}.
            @type visibilityCondition: L{gtk.Widget}
            @param currentCentre: centro Guadalinfo seleccionado inicialmente.
            @type currentCentre: no definido '''

        Log.__init__ (self, where, visibilityCondition, currentCentre)
        self.type = 'Dispatch'

class StatusProgressBar(gtk.ProgressBar):

    ''' B{TODO} (Goomerko's) '''

    def __init__(self):

        ''' B{TODO} (Goomerko's) '''

        gtk.ProgressBar.__init__(self)
        self.event = threading.Event()
        #self.set_text("Trabajando")
        self.reset()

    def reset(self):

        ''' B{TODO} (Goomerko's) '''

        self.fractions =[ele/100.0 for ele in range(0, 110,10)]
        self.actual = 0

    def __in_thread(self):

        ''' B{TODO} (Goomerko's) '''

        gtk.threads_enter()
        self.next_step()
        gtk.threads_leave()
        if self.event.isSet():
            self.event.clear()
            return False
        return True

    def next_step(self):

        ''' B{TODO} (Goomerko's) '''

        self.actual += 1

        if self.actual == len(self.fractions):
            self.actual = 0
            #self.fractions.reverse()

            if self.get_orientation() == gtk.PROGRESS_RIGHT_TO_LEFT:
                self.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
            else:
                self.set_orientation(gtk.PROGRESS_RIGHT_TO_LEFT)

        fraction = self.fractions[self.actual]
        self.set_fraction(fraction)

    def start(self):

        ''' B{TODO} (Goomerko's) '''

        self.event.clear()
        self.reset()
        self.show_all()
        gobject.timeout_add(100, self.__in_thread)

    def stop(self):

        ''' B{TODO} (Goomerko's) '''

        self.event.set()
        self.hide_all()
        while gtk.events_pending():
            gtk.main_iteration()

from common.gui.gui import GraphicalUserInterface
# Fin del archivo.