# -*- coding: utf-8 -*-

''' Interfaz gráfica de usuario. '''

import os
import os.path #@Reimport
import threading
import thread
import time

from thread                import start_new_thread
from sys                   import stdout
from inspect               import getmembers
from Queue                 import Queue

from pygtk                 import require
require ('2.0')
import gtk
import gtk.glade #@Reimport
from pango                 import FontDescription

from Pyro.errors import ProtocolError
from Pyro.errors import ConnectionClosedError
from Pyro.errors import URIError

from common.config         import GLADE_FILE_NAME, ICON_PREFIX, \
                                  APP_ICON_FILE, CONNECTED_ICON_FILE, \
                                  WARNING_ICON_FILE, IMPORT_ICON_FILE, \
                                  EXPORT_ICON_FILE, EDIT_ICON_FILE, CONFIG_ICON_FILE, \
                                  HISTORY_ICON_FILE, ABOUT_ICON_FILE, BUTTON_LABELS, \
                                  WIDGET_MAIN, WIDGET_ABOUT, WIDGET_FILE, \
                                  WIDGET_CONFIG, WIDGET_EDITION, APP_NAME, APP_VERSION, \
                                  APP_AUTHORS, APP_DESCRIPTION, APP_COPYRIGHT, APP_URL, \
                                  APP_URL_LABEL, STATUS_BAR_ONLY_CONTEXT, LOG_CRITICAL, \
                                  LOG_ERROR, LOG_WARNING, LOG_DEBUG, ACT_AS_SERVER, \
                                  PROVINCES, GUI_CONFIG_FILE, INE_CODES_FILE, \
                                  DEFAULT_INTERFACE_CONFIGURATION, \
                                  LIST_OF_CENTRES_FILE_NAME
from common.utils          import read_text_file, write_text_file
from common.utils import ui_connection_safe
from common.gui.files      import parse_GUI_conf_file, dump_GUI_conf_file, Town_name_finder

try:
    from server.daemon.data    import PyroFactory as serverPyroFactory
    from server.config         import LIST_OF_PACKAGES_FILE_NAME, \
                                      LIST_OF_FILES_FILE_NAME
except  Exception, e:
    print "No se importan las bibliotecas del servidor:", str(e)

try:
    from client.daemon.data    import PyroFactory as clientPyroFactory
    from client.daemon.utils import GuadalinfoId
except Exception, e:
    print "No se importan las bibliotecas del cliente", str(e)

def refresh_info (GUI):

    ''' Actualizar la información sobre los centros Guadalinfo.
        @param GUI: interfaz gráfica de usuario que se quiere actualizar.
        @type GUI: L{GraphicalUserInterface} '''

    GUI.info = centros (GUI.nameFinder, GUI.server_ip, GUI.statisticsBar) # , GUI.progressBar)
    #GUI.centres = Centres (GUI.treeviewCentros, GUI.info)

    while True:
        if not GUI.normal_state.isSet():
            GUI.normal_state.wait()

        GUI.refreshingNow = True
        GUI.info = centros (GUI.nameFinder, GUI.server_ip, GUI.statisticsBar) # , GUI.progressBar)
        if GUI.info != {}:
            gtk.threads_enter()
            GUI.centres.populate(GUI.selected, GUI)
            gtk.threads_leave()

        GUI.refreshingNow = False
        i = 0

        while not GUI.forceRefresh and i < GUI.conf ['refresh']:
            time.sleep (1.0)
            i = i + 1

        if GUI.forceRefresh:
            GUI.forceRefresh = False

class GraphicalUserInterface:

    ''' Clase que modela la interfaz gráfica con el usuario.

        Se ocupa de:

         - Cargar la interfaz I{Glade} que está en
           L{common.config.GLADE_FILE_NAME}. Sus iconos se buscan en
           L{common.config.ICON_PREFIX}.
         - Modificar la apariencia y las funcionalidades disponibles en
           función del tipo de interfaz (I{servidor}, I{diputación} o
           I{centro}).
         - Cargar el listado de centros y refrescarlo con la frecuencia
           que se haya configurado. (El archivo de configuración de la
           interfaz se busca en L{common.config.GUI_CONFIG_FILE}).
         - Asociar eventos con manejadores, para responder a las acciones
           del usuario. '''

    PyroFactory = None

    def __init__ (self, role, server_ip):

        ''' Crear un I{GUI} del tipo indicado que obtendrá sus datos de la
            dirección que se especifique.

            C{role} debe ser uno de:

             - L{common.config.ACT_AS_SERVER} (L{None} en realidad) que
               indica una interfaz B{I{servidor}}.
             - Un código de provincia válido (por ejemplo, C{4} o C{11})
               para crear una interfaz de tipo B{I{diputación}}.
             - Un código de centro válido (por ejemplo, C{(11, 429)} o
               C{(41, 16)}), en caso de instanciarse en un B{I{centro
               Guadalinfo}}.

            C{server_ip} debe ser una dirección válida de computador.
            Debe haber un servidor de objetos L{Pyro} en esa máquina.
            Posibles ejemplos:

             - C{192.168.0.1}.
             - C{vaguada-server}.
             - C{localhost}.

            @param role: I{rol} de la interfaz.
            @type role: L{None}, L{int} o L{tuple}
            @param server_ip: dirección del servidor L{Pyro}.
            @type server_ip: L{str}
            @rtype: L{GraphicalUserInterface} '''

        # Cargar la interfaz GTK:
        layout = gtk.glade.XML (GLADE_FILE_NAME, root = WIDGET_MAIN)

        # Asociar todos los «widgets» con este objeto:
        for i in layout.get_widget_prefix (''):
            setattr (self, i.get_name (), i)

        # Insertamos el StatusProgressBar en la interfaz
        statusBox = layout.get_widget("statusBox")
        spb = StatusProgressBar()
        spb.pulse()
        statusBox.pack_start(spb, True, True)
        self.statusProgress = spb

        # Se pone el cursor a «ocupado»:
        self.set_busy (True)

        # Mostrar la ventana principal:
        self.mainWindow.show ()

        # Algunos atributos:
        self.role = role
        self.server_ip = server_ip
        self.lastMessage = []
        self.messages = ''
        self.nameFinder = Town_name_finder (INE_CODES_FILE)
        self.selected = None
        self.path = None
        self.forceRefresh = False

        # Fijar una marca al final en cada campo de texto (registros):
        tmp = self.textviewIncidencias.get_buffer ()
        end = tmp.get_start_iter ()
        tmp.create_mark ('endMark', end, True)
        tmp = self.textviewEnvios.get_buffer ()
        end = tmp.get_start_iter ()
        tmp.create_mark ('endMark', end, True)

        # Activar los «tooltips»:
        self.toolTips = gtk.Tooltips ()
        self.toolTips.enable ()

        # Detalles configurables:
        self.conf = parse_GUI_conf_file (GUI_CONFIG_FILE)

        if self.conf is None or {} == self.conf:
            self.conf = DEFAULT_INTERFACE_CONFIGURATION

        self.applyConfiguration ()

        while gtk.events_pending():
            gtk.main_iteration()

        # Diferencias en el aspecto de los 3 tipos de interfaz:
        if isinstance(self.role, tuple):
            try:
                GraphicalUserInterface.PyroFactory = clientPyroFactory
                self.PyroFactory = clientPyroFactory
            except:
                msg = "No se pudo iniciar la interfaz del cliente. Compruebe"
                msg += " la instalacion de paquete \"vaguada-client-daemon\""
                dialog = gtk.MessageDialog(self.mainWindow, type = gtk.MESSAGE_ERROR, 
                         buttons = gtk.BUTTONS_CLOSE, message_format =  msg)
                dialog.run()
                dialog.destroy()

        else:
            try:
                self.PyroFactory = serverPyroFactory
                uia = self.PyroFactory.get_ui_assistant(self.server_ip)
                print uia
                self.role = uia.get_role()
                GraphicalUserInterface.PyroFactory = serverPyroFactory
            except (ProtocolError, ConnectionClosedError, URIError):
                msg = "No se puede conectar con el demonio"
                dialog = gtk.MessageDialog(self.mainWindow, type = gtk.MESSAGE_ERROR, 
                         buttons = gtk.BUTTONS_CLOSE, message_format =  msg)
                dialog.run()
                dialog.destroy()

        if ACT_AS_SERVER == self.role:
            self.roleBar.push (STATUS_BAR_ONLY_CONTEXT, 'Servidor')
            self.mainWindow.maximize ()
        elif isinstance (self.role, int):
            #index = [i [0] for i in PROVINCES].index (self.role)
            #self.roleBar.push (STATUS_BAR_ONLY_CONTEXT, 'Provincia de ' + PROVINCES [index] [1])
            #self.mainWindow.maximize ()

            # Menú "Archivo":
            self.optionSesion.hide ()
            self.separator1.hide ()
            self.optionImportar.hide ()
            self.optionExportar.hide ()
            self.separator2.hide ()

            # Menú "Editar":
            self.optionCentros.hide ()
            self.optionPaquetes.hide ()
            self.optionArchivos.hide ()
            self.separator5.hide ()

            # Menú "Ver":
            self.optionContraer.hide ()
            self.optionExpandir.hide ()
            self.separator4.hide ()

            # Barra de herramientas:
            self.itemSesion.hide ()
            self.separatortoolitem1.hide ()
            self.separatortoolitem3.hide ()
            self.itemContraer.hide ()
            self.itemExpandir.hide ()

        else:
            index = [i [0] for i in PROVINCES].index (self.role [0])
            self.roleBar.push (STATUS_BAR_ONLY_CONTEXT, '%s (%s)' % \
                               (self.nameFinder.find (self.role), PROVINCES [index] [1]))
            self.cuadroCentros.hide ()

            # Verificar que se puede borrar:
#             self.showRegistros ()

            # Menú "Archivo":
            self.optionSesion.hide ()
            self.separator1.hide ()
            self.optionImportar.hide ()
            self.optionExportar.hide ()
            self.separator2.hide ()

            # Menú "Editar":
            self.optionCentros.hide ()
            self.optionPaquetes.hide ()
            self.optionArchivos.hide ()
            self.separator5.hide ()

            # Menú "Ver":
            #self.toolTips.set_tip (self.optionActualizar, 'Refrescar el estado del centro')
            self.optionContraer.hide ()
            self.optionExpandir.hide ()
            self.separator4.hide ()
            self.optionConectados.hide ()
            self.optionProblematicos.hide ()

            # Barra de herramientas:
            self.itemSesion.hide ()
            self.separatortoolitem1.hide ()
            self.separatortoolitem3.hide ()
            self.toolTips.set_tip (self.buttonActualizar, 'Refrescar el estado del centro [Ctrl+R]')
            self.itemContraer.hide ()
            self.itemExpandir.hide ()
            self.itemConectados.hide ()
            self.itemProblematicos.hide ()

        # Poner un mensaje de inicialización en la barra de tareas:
        message = 'Inicializando ' + APP_NAME + ' ' + APP_VERSION
        self.messages = self.messages + message + '\n'
        self.lastMessage.append (self.statusBar.push (STATUS_BAR_ONLY_CONTEXT, message))
#         self.anotherBar1.push (STATUS_BAR_ONLY_CONTEXT, 'anotherBar1')
#         self.anotherBar2.push (STATUS_BAR_ONLY_CONTEXT, 'anotherBar2')

        # Nombre, versión e icono de la aplicación:
        self.mainWindow.set_title (APP_NAME)
        self.versionBar.push (STATUS_BAR_ONLY_CONTEXT, APP_VERSION)
        self.mainWindow.set_icon_from_file (APP_ICON_FILE)

        # Para abandonar la aplicación:
        self.mainWindow.connect ('delete_event', self.deleteEvent)
        self.mainWindow.connect ('destroy', self.destroy)

        # Conectar «widgets» con eventos:
        layout.signal_autoconnect (self)

        # Conectar elementos de la interfaz con sus manejadores genéricos:

        # Acciones de menú:

        # Acciones de un estado:
        # Menú "Archivo":
        self.optionSesion.connect ('activate', self.__ssh_connection)
        self.optionImportar.connect ('activate', self.manage_list_of_centres, 'Importar')
        self.optionExportar.connect ('activate', self.manage_list_of_centres, 'Exportar')
        self.optionIncidencias.connect ('activate', self.__save_incidences)
        self.optionEnvios.connect ('activate', self.__save_dispatch)
        if isinstance(self.role, int):
            self.optionIncidencias.set_sensitive (False)
            self.optionEnvios.set_sensitive (False)
        self.optionSalir.connect ('activate', gtk.main_quit)
        # Menú "Editar":
        self.optionCentros.connect ('activate', self.edit_listing, 'Centros')
        self.optionPaquetes.connect ('activate', self.edit_listing, 'Paquetes')
        self.optionArchivos.connect ('activate', self.edit_listing, 'Archivos')
        self.optionConfiguracion.connect ('activate', self.configure)
        # Menú "Ver":
        #self.optionActualizar.connect ('activate', self.refresh_now)
        self.optionContraer.connect ('activate', self.contraer)
        self.optionExpandir.connect ('activate', self.expandir)
        self.optionHistorico.connect ('activate', self.edit_listing, 'Historico')
        # Menú "Ayuda":
        self.optionAcerca.connect ('activate', self.optionAcercaActivate)

        # Acciones "biestables":
        # Menú "Archivo":
        self.optionConectados.connect ('toggled', self.toggleSomething, 'Conectados')
        self.optionProblematicos.connect ('toggled', self.toggleSomething, 'Problematicos')
        self.optionRegistros.connect ('toggled', self.toggleSomething, 'Registros')

        # Botones de la barra de herramientas:

        # Acciones de un estado:
        self.buttonSesion.connect ('clicked', self.__ssh_connection)
        self.buttonActualizar.connect ('clicked', self.refresh_now)
        self.buttonContraer.connect ('clicked', self.contraer)
        self.buttonExpandir.connect ('clicked', self.expandir)
        self.buttonContraer.connect ('clicked', self.contraer)
        self.buttonExpandir.connect ('clicked', self.expandir)

        # Acciones "biestables":
        self.buttonConectados.connect ('toggled', self.toggleSomething, 'Conectados')
        self.buttonProblematicos.connect ('toggled', self.toggleSomething, 'Problematicos')
        self.buttonRegistros.connect ('toggled', self.toggleSomething, 'Registros')

        # Árbol de centros:
        self.treeviewCentros.connect ('row_activated', self.rowActivated)
        self.treeviewCentros.connect ('cursor-changed', self.cursorChanged)

        # Botones del panel de estado:
        children = self.arrayEstado.get_children ()

        # Hack: Esta propiedad está concebida para que se pueda acceder al
        # método set_busy desde la función show_status del módulo
        # common.gui.status
        self.arrayEstado.gui_parent = self
        self.normal_state = threading.Event()
        self.normal_state.set()

        #for i in range (len (children)):
        #    inner = children [i].get_children ()

        #    for j in range (len (inner [3].get_children ())):
        #        inner[3].get_children () [j].connect ('clicked',
        #                self.solve_incidence, i, j, self.server_ip)

        for inner in children:
            inner_obj = inner.get_children()
            inner_children = inner_obj[3].get_children()

            for inner_child in inner_children:
                inner_child.connect('clicked',
                        self.solve_incidence,
                        children.index(inner),
                        inner_children.index(inner_child),
                        self.server_ip)


        # Campos de texto de los registros:
#         self.textviewIncidencias.get_buffer ().connect ('changed', scroll_down, self.scrolledwindowIncidencias)
#         self.textviewEnvios.get_buffer ().connect ('changed', scroll_down, self.scrolledwindowEnvios)

        self.notebookRegistros.set_property ('homogeneous', True)
#        self.notebookRegistros.set_property ('enable-popup', False)

        # Activar imágenes en botones (aunque debe estar por defecto):
        self.settings = gtk.Settings ()
        self.settings.set_property ('gtk-button-images', True)

        # Crear e inicializar los objetos que representan componentes de la interfaz
        # (tambien cambian en funcion de si somos servidores o clientes):

        if ACT_AS_SERVER == self.role or isinstance(self.role, int):

            # TODO. Esto es provisional:
            #self.info = municipiosAleatorios (40, progressBar = self.progressBar)
            #self.info = centros (self.server_ip, progressBar = self.progressBar)
            self.centres = Centres (self.treeviewCentros, {})
            refresh_thread = start_new_thread (refresh_info, (self,))

            self.status = Status (self.arrayEstado, self.arrayEstado, self.server_ip, self.labelEstado)
            self.incidenceLog = IncidenceLog (self.textviewIncidencias, self.notebookRegistros, self.server_ip)
            self.dispatchLog = DispatchLog (self.textviewEnvios, self.notebookRegistros, self.server_ip)
        #elif isinstance (self.role, int):
        #    # Provincial

        #    # TODO. Esto es provisional:
        #    self.info = municipiosAleatorios (15, progressBar = self.progressBar, province = self.role)

        #    self.centres = Centres (self.treeviewCentros, self.info, province = self.role)
        #    self.status = Status (self.arrayEstado, self.arrayEstado, self.server_ip, self.labelEstado)
        #    self.incidenceLog = IncidenceLog (self.textviewIncidencias, self.notebookRegistros, self.server_ip)
        #    self.dispatchLog = DispatchLog (self.textviewEnvios, self.notebookRegistros, self.server_ip)
        else:
            # Municipal
            self.info = municipioConcreto (self.role)
            self.status = Status (self.arrayEstado, self.arrayEstado, self.server_ip, self.labelEstado)

            #Obtenemos el código de este centro
            gid = GuadalinfoId()
            self.selected = (int(gid['ID_PROVINCIA']), int(gid['ID_LOCALIDAD']))

            self.status.change(self.selected, True)
            self.__init_change_thread()

            # self.incidenceLog = IncidenceLog (self.textviewIncidencias, self.notebookRegistros, self.server_ip, self.role)
            self.incidenceLog = IncidenceLog (self.textviewIncidencias, self.notebookRegistros, self.server_ip)
            self.incidenceLog.change(self.selected, True)
            # self.dispatchLog = DispatchLog (self.textviewEnvios, self.notebookRegistros, self.server_ip, self.role)
            self.dispatchLog = DispatchLog (self.textviewEnvios, self.notebookRegistros, self.server_ip)
            self.dispatchLog.change(self.selected, True)

            #self.showRegistros ()

        # Borrar el mensaje de inicialización en la barra de tareas:
        self.statusBar.remove (STATUS_BAR_ONLY_CONTEXT, self.lastMessage.pop ())

        # Se pone de nuevo el cursor normal:
        self.set_busy (False)

    def __init_change_thread (self):

        ''' Iniciar el ciclo de actualización de los datos (¿no, Gúmer?) '''

        def in_thread():

            while True:
                time.sleep(60)
                gtk.threads_enter()
                self.status.change(self.selected, True)
                self.incidenceLog.update()
                self.dispatchLog.update()
                gtk.threads_leave()

        thread.start_new_thread(in_thread,())

    def solve_incidence (self, widget, i, j, server_ip):

        ''' Solucionar una incidencia activa.
            @param widget: sin utilidad.
            @type widget: irrelevante
            @param i: incidencia.
            @type i: L{int}
            @param j: método de solución de la incidencia.
            @type j: L{int}
            @param server_ip: dirección IP del servidor.
            @type server_ip: L{string} '''

        self.set_busy(True)
        self.normal_state.clear()

        event = threading.Event()
        thread.start_new_thread(solve,(self, i, j, server_ip, event))

        while not event.isSet():
            while gtk.events_pending():
                gtk.main_iteration()
            time.sleep(0.1)

        self.set_busy(False)
        self.normal_state.set()

    def main (self):

        ''' Bucle principal I{GTK}.

            Inicializar las hebras de GTK e iniciar su bucle principal. Se trata
            del bucle que constituye el funcionamiento normal de la aplicación, y
            del que sólo se sale cerrando I{Vaguada}. '''

        gtk.threads_init ()
        gtk.main ()

    def deleteEvent (self, widget, event):

        ''' Manejador para el evento L{delete_event}.

            Se podría utilizar para mostrar un mensaje de confirmación al usuario
            antes de salir de la aplicación. B{Actualmente no se usa}.

            @param widget: el control GTK que recibió la señal.
            @type widget: L{gtk.Widget}
            @param event: el evento GTK que causó la señal.
            @type event: L{gtk.Event}
            @return: si la aplicación debe seguir abierta, o no.
            @rtype: L{bool} '''

        # Este manejador debe devolver «False» para propagar el evento hacia adelante.
        # Se podría devolver «True» para evitar que se cerrase la aplicación.

        return False

    def destroy (self, widget, data = None):

        ''' Manejador para el evento L{destroy}.

            Se podría utilizar para mostrar un mensaje de confirmación al usuario
            antes de salir de la aplicación. B{Actualmente no se usa}.

            @param widget: el control GTK que recibió la señal.
            @type widget: L{gtk.Widget}
            @param data: el evento GTK que causó la señal.
            @type data: L{event} '''

        gtk.main_quit ()

    def optionAcercaActivate (self, widget):

        ''' Invocar al cuadro de diálogo I{Acerca de...}
            Se cambian algunas propiedades del cuadro de diálogo (rótulos, icono, enlace)
            para adaptarlo a las características de la aplicación.

            @param widget: sin utilidad.
            @type widget: irrelevante '''

        gtk.about_dialog_set_url_hook (self.followLink)
        gtk.about_dialog_set_email_hook (self.followLink)
        layout = gtk.glade.XML (GLADE_FILE_NAME, root = WIDGET_ABOUT)
        about = layout.get_widget (WIDGET_ABOUT)
        about.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        about.set_icon_from_file (ABOUT_ICON_FILE)
        logo = gtk.gdk.pixbuf_new_from_file_at_size (APP_ICON_FILE, 60, 83)
        about.set_logo (logo)
        about.set_name (APP_NAME)
        about.set_version (APP_VERSION)
        about.set_comments (APP_DESCRIPTION)
        about.set_copyright (APP_COPYRIGHT)
        about.set_website (APP_URL)
        about.set_website_label (APP_URL_LABEL)
        about.set_authors (APP_AUTHORS)
        about.run ()

    def toggleSomething (self, widget, name):

        ''' Cambiar el estado de un control biestble (botón u opción de menú).
            El parámetro L{name} indica la acción, y debe ser C{'Conectados'},
            C{'Problematicos'} o C{Registros}.

            @param widget: el control GTK que recibió la señal.
            @type widget: L{gtk.Widget}
            @param name: acción a realizar.
            @type name: L{str} '''

        # Se pone el cursor a «ocupado»:
        self.set_busy (True)
        self.normal_state.clear()

        if 'Conectados' == name or 'Problematicos' == name:
            self.centres.unmark ()
            while gtk.events_pending():
                gtk.main_iteration()
            self.status.change (None)
            self.incidenceLog.change (None)
            self.dispatchLog.change (None)

            if ACT_AS_SERVER == self.role:
                self.optionIncidencias.set_sensitive (False)
                self.optionEnvios.set_sensitive (False)
                self.buttonSesion.set_sensitive (False)

            if 'Conectados' == name:

                if widget.get_active ():
                    self.showConectados ()
                else:
                    self.hideConectados ()

            else:

                if widget.get_active ():
                    self.showProblematicos ()
                else:
                    self.hideProblematicos ()

        elif 'Registros' == name:

            if widget.get_active ():
                self.showRegistros ()
            else:
                self.hideRegistros ()

        # Se pone de nuevo el cursor normal:
        self.set_busy (False)
        self.normal_state.set()

    def contraer (self, foo):

        ''' Contraer completamente el listado de centros Guadalinfo. '''

        # Se pone el cursor a «ocupado»:
        self.set_busy (True)

        self.centres.collapse ()

        # Se pone de nuevo el cursor normal:
        self.set_busy (False)

    def expandir (self, foo):

        ''' Expandir completamente el listado de centros Guadalinfo. '''

        # Se pone el cursor a «ocupado»:
        self.set_busy (True)

        self.centres.expand ()

        # Se pone de nuevo el cursor normal:
        self.set_busy (False)

    def manage_list_of_centres (self, widget, name):

        ''' Importar o exportar el listado de centros Guadalinfo.
            Se invoca al cuadro de diálogo correspondiente, cambiándolo para que se
            adapte a la acción seleccionada, y se actúa en consecuencia si el usuario
            indica un archivo válido. El parámetro L{name} indica la acción, y debe
            ser C{'Importar'} o C{'Exportar'}.

            @param widget: sin utilidad.
            @type widget: irrelevante
            @param name: acción a realizar.
            @type name: L{str} '''

        layout = gtk.glade.XML (GLADE_FILE_NAME, root = WIDGET_FILE)
        window = layout.get_widget (WIDGET_FILE)
        window.set_position (gtk.WIN_POS_CENTER_ON_PARENT)

        if 'Importar' == name:
            window.set_action (gtk.FILE_CHOOSER_ACTION_OPEN)
            window.set_title ('Archivo desde el que importar')
            window.set_icon_from_file (IMPORT_ICON_FILE)
            result = window.run ()

            if result in (gtk.RESPONSE_ACCEPT, gtk.RESPONSE_OK, gtk.RESPONSE_YES, gtk.RESPONSE_APPLY):
                window.hide ()
                message = 'Importando listado de centros desde «' + str (window.get_uri ()) + '»'
                self.messages = self.messages + message + '\n'
                self.lastMessage.append (self.statusBar.push (STATUS_BAR_ONLY_CONTEXT, message))

                while gtk.events_pending ():
                    gtk.main_iteration ()

                self.statusBar.remove (STATUS_BAR_ONLY_CONTEXT, self.lastMessage.pop ())
            else:
                window.hide ()

                while gtk.events_pending ():
                    gtk.main_iteration ()

        else:
            window.set_action (gtk.FILE_CHOOSER_ACTION_SAVE)
            window.set_title ('Archivo al que exportar')
            window.set_icon_from_file (EXPORT_ICON_FILE)
            result = window.run ()

            if result in (gtk.RESPONSE_ACCEPT, gtk.RESPONSE_OK, gtk.RESPONSE_YES, gtk.RESPONSE_APPLY):
                window.hide ()
                message = 'Exportando listado de centros a «' + str (window.get_uri ()) + '»'
                self.messages = self.messages + message + '\n'
                self.lastMessage.append (self.statusBar.push (STATUS_BAR_ONLY_CONTEXT, message))

                while gtk.events_pending ():
                    gtk.main_iteration ()

                self.statusBar.remove (STATUS_BAR_ONLY_CONTEXT, self.lastMessage.pop ())
            else:
                window.hide ()

                while gtk.events_pending ():
                    gtk.main_iteration ()

    def configure (self, foo):

        ''' Configurar la interfaz de usuario, invocando su cuadro de diálogo. '''

        layout = gtk.glade.XML (GLADE_FILE_NAME, root = WIDGET_CONFIG)
        window = layout.get_widget (WIDGET_CONFIG)
        window.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        widget = {}

        for i in layout.get_widget_prefix (''):
            widget [i.get_name ()] = i

        window.set_icon_from_file (CONFIG_ICON_FILE)
        window.set_title ('Configuración de ' + APP_NAME)
        widget ['buttonLabels'].set_active (self.conf ['labels'])
        widget ['refreshTimeout'].set_value (self.conf ['refresh'])
        widget ['fontSize'].set_value (self.conf ['font'])
        result = window.run ()

        # Se pone el cursor a «ocupado»:
        self.set_busy (True)

        if result in (gtk.RESPONSE_ACCEPT, gtk.RESPONSE_OK, gtk.RESPONSE_YES, gtk.RESPONSE_APPLY):

            if widget ['buttonLabels'].get_active ():
                self.conf ['labels'] = True
            else:
                self.conf ['labels'] = False

            self.conf ['refresh'] = int (widget ['refreshTimeout'].get_value ())
            self.conf ['font'] = int (widget ['fontSize'].get_value ())
            self.applyConfiguration ()
            resultCode = dump_GUI_conf_file (GUI_CONFIG_FILE, self.conf)
            self.forceRefresh = True

            if resultCode:
                message = 'Guardando la configuración en «' + GUI_CONFIG_FILE + '»'
                self.messages = self.messages + message + '\n'
                self.lastMessage.append (self.statusBar.push (STATUS_BAR_ONLY_CONTEXT, message))

                while gtk.events_pending ():
                    gtk.main_iteration ()

                self.statusBar.remove (STATUS_BAR_ONLY_CONTEXT, self.lastMessage.pop ())
            else:
                message = 'No se ha podido escribir en «' + GUI_CONFIG_FILE + '»'
                self.messages = self.messages + message + '\n'
                self.lastMessage.append (self.statusBar.push (STATUS_BAR_ONLY_CONTEXT, message))

                while gtk.events_pending ():
                    gtk.main_iteration ()

                self.statusBar.remove (STATUS_BAR_ONLY_CONTEXT, self.lastMessage.pop ())

            while gtk.events_pending ():
                gtk.main_iteration ()

        window.hide ()

        # Se pone de nuevo el cursor normal:
        self.set_busy (False)

    def refresh_now (self, foo):

        ''' Actualizar la interfaz (su listado de centros) B{ahora}.

            @param foo: sin utilidad.
            @type foo: irrelevante '''

        self.forceRefresh = True

    def edit_listing (self, widget, name):

        """ Manejador genérico para eventos que invoquen al cuadro de diálogo C{editionDialog}.

            La edición de cualquiera de los 3 listados (I{centros}, I{paquetes} o I{archivos})
            y la visualización del histórico de mensajes de la barra de estado se realiza
            mostrando el cuadro de diálogo C{editionDialog} y modificando un poco su
            apariencia y comportamiento.

            Las combinaciones de eventos y controles que llaman a este manejador son las
            siguientes (se incluye también el valor del parámetro C{name} asociado):
             - C{activate} sobre L{self.optionCentros} (C{Centros}).
             - C{activate} sobre L{self.optionPaquetes} (C{Paquetes}).
             - C{activate} sobre L{self.optionArchivos} (C{Archivos}).
             - C{activate} sobre L{self.optionHistorico} (C{Historico}).

            @param widget: control GTK que provocó el evento.
            @type widget: C{gtk.Widget}
            @param name: cadena que identifica la acción.
            @type name: L{str} """

        layout = gtk.glade.XML (GLADE_FILE_NAME, root = WIDGET_EDITION)
        window = layout.get_widget (WIDGET_EDITION)
        window.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        widget = {}

        for i in layout.get_widget_prefix (''):
            widget [i.get_name ()] = i

        # Se aplica la configuración actual de la fuente:
        font_description = FontDescription ('monospace ' + str (self.conf ['font']))
        widget ['textView'].modify_font (font_description)

        if 'Historico' == name:
            window.set_icon_from_file (HISTORY_ICON_FILE)
            window.set_title ('Histórico de mensajes')
            widget ['buttonCancelar'].hide ()
            widget ['textView'].set_editable (False)
            widget ['textView'].get_buffer ().set_text (self.messages)
        else:
            window.set_icon_from_file (EDIT_ICON_FILE)
            widget ['buttonCancelar'].show ()
            widget ['textView'].set_editable (True)
            window.set_title ('Edición del listado de ' + name.lower ())
            uia = self.PyroFactory.get_ui_assistant (self.server_ip)

            if 'Centros' == name:
                text = uia.get_centres ()
            elif 'Paquetes' == name:
                text = uia.get_packages ()
            else:
                text = uia.get_files ()

            if text is not None:
                widget ['textView'].get_buffer ().set_text (text)
            else:
                widget ['textView'].get_buffer ().set_text ('[Error]')
                        
        result = window.run ()

        if result in (gtk.RESPONSE_ACCEPT, gtk.RESPONSE_OK, gtk.RESPONSE_YES, gtk.RESPONSE_APPLY):
            window.hide ()

            if 'Historico' != name:

                if 'Centros' == name:
                    message = 'Actualizando el listado de ' + name.lower () + ' en «' + \
                              LIST_OF_CENTRES_FILE_NAME + '»'
                elif 'Paquetes' == name:
                    message = 'Actualizando el listado de ' + name.lower () + ' en «' + \
                              LIST_OF_PACKAGES_FILE_NAME + '»'
                else:
                    message = 'Actualizando el listado de ' + name.lower () + ' en «' + \
                              LIST_OF_FILES_FILE_NAME + '»'

                self.messages = self.messages + message + '\n'
                self.lastMessage.append (self.statusBar.push (STATUS_BAR_ONLY_CONTEXT, message))

                while gtk.events_pending ():
                    gtk.main_iteration ()

                text_buff = widget ['textView'].get_buffer ()
                begin = text_buff.get_start_iter ()
                end = text_buff.get_end_iter ()
                text = text_buff.get_text (begin, end)
                uia = self.PyroFactory.get_ui_assistant (self.server_ip)

                if 'Centros' == name:
                    uia.set_centres (text)
                elif 'Paquetes' == name:
                    uia.set_packages (text)
                else:
                    uia.set_files (text)

                self.statusBar.remove (STATUS_BAR_ONLY_CONTEXT, self.lastMessage.pop ())

        else:
            window.hide ()

            while gtk.events_pending ():
                gtk.main_iteration ()

    edit_listing = ui_connection_safe(edit_listing)

    def showConectados (self):

        ''' Mostrar B{sólo} los centros conectados en el listado de centros.
            O sea, puede que no se muestren B{todos} los centros Guadalinfo. '''

        self.optionConectados.set_active (True)
        self.buttonConectados.set_active (True)
        self.centres.showOnlyConnected (True)

    def hideConectados (self):

        ''' No mostrar B{sólo} los centros conectados en el listado de centros.
            O sea, mostrar los centros Guadalinfo independientemente de su actividad
            actual. '''

        self.optionConectados.set_active (False)
        self.buttonConectados.set_active (False)
        self.centres.showOnlyConnected (False)

    def showProblematicos (self):

        ''' Mostrar B{sólo} los centros problemáticos en el listado de centros.
            O sea, puede que no se muestren B{todos} los centros Guadalinfo. '''

        self.optionProblematicos.set_active (True)
        self.buttonProblematicos.set_active (True)
        self.centres.showOnlyProblematic (True)

    def hideProblematicos (self):

        ''' No mostrar B{sólo} los centros problemáticos en el listado de centros.
            O sea, mostrar los centros Guadalinfo independientemente de su número de
            incidencias activas. '''

        self.optionProblematicos.set_active (False)
        self.buttonProblematicos.set_active (False)
        self.centres.showOnlyProblematic (False)

    def showRegistros (self):

        ''' Mostrar los registros de incidencias y envíos en la interfaz. '''

        self.optionRegistros.set_active (True)
        self.buttonRegistros.set_active (True)
        self.incidenceLog.update ()
        self.dispatchLog.update ()
        self.notebookRegistros.show ()

    def hideRegistros (self):

        ''' Ocultar los registros de incidencias y envíos de la interfaz. '''

        self.optionRegistros.set_active (False)
        self.buttonRegistros.set_active (False)
        self.incidenceLog.update ()
        self.dispatchLog.update ()
        self.notebookRegistros.hide ()

    def followLink (dialog, link, user_data):

        ''' Seguir el enlace que hay en el cuadro de diálogo I{Acerca de...}
            Este método es necesario para que al hacer I{clic} sobre el enlace que hay
            en el cuadro de diálogo I{Acerca de...} se abra la dirección deseada en un
            navegador I{web}. B{Actualmente no se usa}.
            @param link: hiperenlace.
            @type link: ¿L{str}?
            @param user_data: sin utilidad.
            @type user_data: irrelevante '''

        pass

    def deactivateRows (self):

        ''' No seleccionar ninguna fila del listado de centros. '''

        self.centres.unmark ()
        self.path = None

    def cursorChanged (self, foo):

        ''' Activar una nueva fila del listado de centros al cambiar el cursor. '''

        path = self.treeviewCentros.get_cursor () [0]

        while gtk.events_pending():
            gtk.main_iteration()

        # Se pone el cursor ocupado:
        self.set_busy (True)

        self.rowActivated (self.treeviewCentros, path)

        # Se pone de nuevo el cursor normal:
        self.set_busy (False)

    def rowActivated (self, widget, path, view_column = None, force = True):

        ''' Activar una fila determinada del listado de centros.

            @param widget: el control que causó la activación (el árbol de centros).
            @type widget: L{gtk.Widget}
            @param path: el I{camino} por el árbol hasta la fila que se activó.
            @type path: L{path}
            @param view_column: sin utilidad.
            @type view_column: irrelevante
            @param force: indica si debe forzarse una actualización en el resto de
                          controles, o no.
            @type force: L{bool} '''

        # Se pone el cursor a «ocupado»:
        self.normal_state.clear()

        if 'treeviewCentros' == widget.name:
            model = widget.get_model ()
            iter = model.get_iter (path)
            codPro = int (model.get_value (iter, 0))
            codMun = int (model.get_value (iter, 1))
            self.centres.unmark ()
            self.selected = (codPro, codMun)
            self.path = path

            if (-1, -1) == self.selected:
                self.status.change (None, force)
                self.incidenceLog.change (None, force)
                self.dispatchLog.change (None, force)

                if isinstance(self.role, int):
                    self.optionSesion.set_sensitive (False)
                    self.optionIncidencias.set_sensitive (False)
                    self.optionEnvios.set_sensitive (False)
                    self.buttonSesion.set_sensitive (False)
                    self.arrayEstado.set_sensitive (False)

                    for i in self.arrayEstado.get_children ():
                        i.hide ()

            else:
                self.centres.mark (path)
                self.status.change (self.selected, force)
                self.incidenceLog.change (self.selected, force)
                self.dispatchLog.change (self.selected, force)

                if ACT_AS_SERVER == self.role:
                    try:
                        connected = int(model.get_value(iter, 3))
                    except TypeError:
                        connected = 0
                    if connected == 1:
                        self.buttonSesion.set_sensitive (True)
                        self.optionSesion.set_sensitive (True)
                    else:
                        self.buttonSesion.set_sensitive (False)
                        self.optionSesion.set_sensitive (False)


                self.optionIncidencias.set_sensitive (True)
                self.optionEnvios.set_sensitive (True)


        else:
            self.log (LOG_DEBUG, widget.name)

        self.normal_state.set()

    def showButtonLabels (self, value):

        ''' Establecer si se muestra texto en los iconos de la barra de herramientas.

            @param value: indica si se muestra texto, o no.
            @type value: L{bool} '''

        tmp = bool (value)
        attribute = getmembers (self)
        names = [i [0] for i in attribute]

        for i in BUTTON_LABELS:
            index = names.index (i)

            if tmp:
                attribute [index] [1].show ()
            else:
                attribute [index] [1].hide ()

    def applyConfiguration (self):

        ''' Leer L{conf} y aplicar esa configuración a la interfaz. '''

        # 1. Etiquetas en la barra de herramientas ("labels"):

        if self.conf ['labels']:
            self.showButtonLabels (True)
        else:
            self.showButtonLabels (False)


        # 3. Tamaño de la fuente para registros e info sobre incidencias ("font"):
        font_description = FontDescription ('monospace ' + str (self.conf ['font']))
        self.textviewIncidencias.modify_font (font_description)
        self.textviewEnvios.modify_font (font_description)
        inc = self.arrayEstado.get_children ()

        for i in inc:
            i.get_children () [2].modify_font (font_description)

    def log (self, log_level, message):

        ''' Hacer una anotación en el registro (I{log}) de la interfaz.
            Cada entrada en el I{log} es un texto (L{message}) con un I{nivel de log}
            asociado (L{log_level}). L{log_level} debe ser uno de:
            L{common.config.LOG_DEBUG}, L{common.config.LOG_WARNING},
            L{common.config.LOG_ERROR}, L{common.config.LOG_CRITICAL}.

            @param log_level: nivel de I{log}.
            @type log_level: L{int}
            @param message: texto a anotar.
            @type message: L{str} '''

        stream = stdout

        if LOG_DEBUG == log_level:
            stream.write (APP_NAME + ': depuracion: ' + str (message) + '.\n')
        elif LOG_WARNING == log_level:
            stream.write (APP_NAME + ': aviso: ' + str (message) + '.\n')
        elif LOG_ERROR == log_level:
            stream.write (APP_NAME + ': ERROR: ' + str (message) + '.\n')
        elif LOG_CRITICAL == log_level:
            stream.write (APP_NAME + ': CRITICO: ¡' + str (message) + '!\n')
        else:
            self.log (LOG_DEBUG, 'el nivel de "log" ' + str (log_level) + ' no es válido')

    def set_busy (self, value):

        ''' Poner el cursor de la aplicación a I{ocupado}.

            @param value: indica si debe mostrarse el progreso de la tarea actual en la
                          barra de progreso (¿no, Gúmer?)
            @type value: L{bool} '''

        if value:
            #self.statusProgress.start()
            self.watchCursor = gtk.gdk.Cursor (gtk.gdk.WATCH)
            self.mainWindow.window.set_cursor (self.watchCursor)
            self.mainWindow.set_sensitive(False)
        else:
            #self.statusProgress.stop()
            self.mainWindow.window.set_cursor (None)
            self.mainWindow.set_sensitive(True)
        
    def __ssh_connection (self, foo):

        ''' Iniciar una sesión remota en el centro actualmente seleccionado. '''

        def manage_connection(param):
            message = "Conectando con el centro..."
            self.messages = self.messages + message + '\n'
            self.lastMessage.append (self.statusBar.push (STATUS_BAR_ONLY_CONTEXT, message))

            gtk.threads_enter()

            # Se pone el cursor a «ocupado»:
            self.set_busy(True)

            gtk.threads_leave()

            # Task
            uia = self.PyroFactory.get_ui_assistant (self.server_ip)
            centre_ip = uia.start_ssh(self.selected)

            # Se pone de nuevo el cursor normal:
            gtk.threads_enter()
            self.set_busy(False)

            self.statusBar.remove (STATUS_BAR_ONLY_CONTEXT, self.lastMessage.pop ())
            gtk.threads_leave()

            os.system("gnome-terminal -e \"ssh soporte@%s\"" % centre_ip)
            print "########## Centro:", centre_ip
            #uia = self.PyroFactory.get_ui_assistant (self.server_ip)
            #uia.end_ssh (self.selected)

        manage_connection = ui_connection_safe(manage_connection)

        start_new_thread(manage_connection, (None,))
        while gtk.events_pending():
            gtk.main_iteration()

    __ssh_connection = ui_connection_safe(__ssh_connection)

    def __save_incidences(self, *params):

        ''' Guardar un registro de incidencias. '''

        def save(filepath, queue):
            uia = self.PyroFactory.get_ui_assistant(self.server_ip)
            content = uia.get_inc_log(self.selected)
            fileobj = open(filepath, 'w')
            fileobj.write(content)
            fileobj.close()

            queue.put(None)

        filepath = self.__save_dialog()
        if filepath:
            queue = Queue()
            thread.start_new_thread(save, (filepath, queue))

            self.set_busy(True)
            while queue.empty():
                while gtk.events_pending():
                    gtk.main_iteration()
            self.set_busy(False)

    def __save_dialog (self):

        ''' Presentar un cuadro de diálogo genérico para guardar en un archivo
            (¿no, Gúmer?) '''

        if self.selected and (-1, -1) != self.selected:

            filepath = None
            dialog = gtk.FileChooserDialog(title = "Guardar...",
                    action = gtk.FILE_CHOOSER_ACTION_SAVE, 
                    buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
                            gtk.STOCK_SAVE, gtk.RESPONSE_OK))
            filter = gtk.FileFilter()
            filter.set_name('Todos los ficheros')
            filter.add_pattern('*')
            dialog.add_filter(filter)

            dialog.set_default_response(gtk.RESPONSE_OK)

            while True:
                response = dialog.run()
                if response == gtk.RESPONSE_OK:
                    filepath = dialog.get_filename()
                    if os.path.exists(filepath):
                        question = gtk.MessageDialog(dialog, gtk.DIALOG_MODAL, 
                                gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
                                "El fichero existe, ¿sobreescribir?")
                        if question.run() == gtk.RESPONSE_NO:
                            question.destroy()
                            continue
                        question.destroy()
                    break
                elif response == gtk.RESPONSE_CANCEL:
                    break

            dialog.destroy()

            return filepath

    def __save_dispatch(self, *params):

        ''' Guardar un registro de envíos. '''

        def save(filepath, queue):
            uia = self.PyroFactory.get_ui_assistant(self.server_ip)
            content = uia.get_sync_log(self.selected)
            fileobj = open(filepath, 'w')
            fileobj.write(content)
            fileobj.close()

            queue.put(None)

        filepath = self.__save_dialog()
        if filepath:
            queue = Queue()
            thread.start_new_thread(save, (filepath, queue))

            self.set_busy(True)
            while queue.empty():
                while gtk.events_pending():
                    gtk.main_iteration()
            self.set_busy(False)

from common.gui.components import Centres, Status, IncidenceLog, DispatchLog, \
                                  StatusProgressBar
from common.gui.misc       import municipioConcreto, centros, scroll_down
from common.gui.status     import solve

