# -*- coding: utf-8 -*-

''' Biblioteca de funciones misceláneas. '''

from random             import randint, choice

from pygtk              import require
import gtk
require ('2.0')
from gtk                import events_pending, main_iteration, Adjustment, \
                               threads_enter, threads_leave

from common.config      import PROVINCES, STATUS_BAR_ONLY_CONTEXT

def centros (nameFinder, server_ip, statisticsBar, progressBar = None):

    ''' Construir una estructura que contenga todos los centros y sus estados correspondientes.

        @param nameFinder: objeto capaz de devolver nombres de municipios a partir de
                           sus códigos INE.
        @type nameFinder: L{common.gui.files.Town_name_finder}
        @param server_ip: dirección IP del servidor.
        @type server_ip: L{str}
        @param statisticsBar: barra de estado donde escribir algunas estadísticas.
        @type statisticsBar: L{gtk.StatusBar}
        @param progressBar: barra de progreso en la que ir reflejando el avance de la tarea
                            mientras se obtiene y construye el listado de centros.
        @type progressBar: L{gtk.ProgressBar}
        @return: estructura de datos conteniendo el listado de centros y su estado actual
                 (actividad, incidencias, etc.).
        @rtype: L{dict} de L{dict} etc. '''

    if progressBar is not None:
        threads_enter ()
        progressBar.set_fraction (0.0)
        progressBar.set_text ('')
        progressBar.show ()
        threads_leave ()

    try:
        uia = GraphicalUserInterface.PyroFactory.get_ui_assistant (server_ip)
        cd = uia.get_all_centre_data ()
    except:
        return  {}
    result = {}
    size = len (cd.keys ())
    count = 0

    for i in PROVINCES:
        result [str (i [0])] = {}

    conn = 0
    warn = 0
    no_of_inc = 0

    for i in cd.keys ():
        tmp = {}
        index = [j [0] for j in PROVINCES].index (i [0])
        aux = PROVINCES [index]
        tmp ['provCode'] = aux [0]
        tmp ['provName'] = aux [1]
        tmp ['munCode'] = i [1]
        mun_name = nameFinder.find (i)

        if mun_name is None:
            mun_name = 'Municipio #' + str (tmp ['munCode'])

        tmp ['munName'] = mun_name
        tmp ['conn'] = cd [i] ['conn']
        tmp ['comm'] = cd [i] ['comm']
        tmp ['warn'] = cd [i] ['ninc']
        tmp ['last'] = cd [i] ['last_conn']

        if tmp ['warn'] > 0:
            tmp ['alert'] = True
        else:
            tmp ['alert'] = False

        result [str (aux [0])] [str (i [1])] = tmp

        if tmp ['conn']:
            conn += 1

        if tmp ['alert']:
            warn += 1

        no_of_inc += tmp ['warn']

        if progressBar is not None:
            threads_enter ()
            progressBar.set_fraction (float (count + 1) / size)
            progressBar.set_text (str (count + 1) + '/' + str (size))
            threads_leave ()
            count = count + 1

        threads_enter()
        while events_pending ():
            main_iteration ()
        threads_leave()

    if 0 == len (cd):
        ratio1 = 0
    else:
        ratio1 = conn * 100 / len (cd)

    if 0 == conn:
        ratio2 = 0.0
    else:
        ratio2 = float (no_of_inc) / conn

    statistics = '%i/%i (%i%%) con., %i inc. (media %.1f en %i centros)' % \
                 (conn, len (cd), ratio1, no_of_inc, ratio2, warn)
    gtk.threads_enter()
    statisticsBar.pop (STATUS_BAR_ONLY_CONTEXT)
    statisticsBar.push (STATUS_BAR_ONLY_CONTEXT, statistics)
    gtk.threads_leave()

    if progressBar is not None:
        threads_enter ()
        progressBar.hide ()
        progressBar.set_fraction (0.0)
        progressBar.set_text ('')
        threads_leave ()

    return result

def municipioConcreto (role):

    ''' Generar un listado que contenga un único centro concreto.
        Para la versión I{cliente} de I{Vaguada}.

        @param role: un par I{(código_prov, código_mun)} que identifica al centro.
        @type role: L{list} de L{int}
        @return: una estructura conteniendo información sobre ese centro.
        @rtype: L{dict} de L{dict} etc. '''

    result = {}

    for i in PROVINCES:
        result [str (i [0])] = {}

    tmp = {}
    tmp ['provCode'] = role [0]
    tmp ['provName'] = PROVINCES [[i [0] for i in PROVINCES].index (role [0])]
    tmp ['munCode'] = role [1]
    tmp ['munName'] = 'Municipio #' + str (tmp ['munCode'])
    tmp ['conn'] = choice ((True, False))

    if choice ((True, False)):
        tmp ['warn'] = randint (1, 10)
        tmp ['alert'] = True
    else:
        tmp ['warn'] = 0
        tmp ['alert'] = False

    activity = randint (0, 7)

    if 0 == activity:
        tmp ['act'] = 'Recibiendo'
    elif 1 == activity:
        tmp ['act'] = 'Enviando'
    elif 2 == activity:
        tmp ['act'] = 'Sincronizando'
    else:
        tmp ['act'] = None

    if 0 == randint (0, 3):
        tmp ['comm'] = 'Esto es un comentario acerca de este centro Guadalinfo.'
    else:
        tmp ['comm'] = None

    result [str (role [0])] [str (role [1])] = tmp

    return result

def scroll_down (widget, container):

    ''' Desplazar el L{container} hasta abajo del todo.

        @param widget: control que causó el desplazamiento (no se usa).
        @type widget: L{gtk.Widget}
        @param container: El control que contiene a éste, y que se
                          va a desplazar.
        @type container: L{gtk.Widget} '''

    horizontalAdjustment = container.get_hscrollbar ().get_adjustment ()
    verticalAdjustment = container.get_vscrollbar ().get_adjustment ()
    maxScroll = verticalAdjustment.upper - verticalAdjustment.page_size
    horizontalAdjustment.set_value (0.0)
    verticalAdjustment.set_value (maxScroll)

from common.gui.gui import GraphicalUserInterface

# Fin del archivo.

