# -*- coding: utf-8 -*-

''' Manejo del panel de estado (el listado de incidencias). '''

import gtk
import thread
import time
import traceback
import sys

from Queue import Queue
from gtk                import Tooltips
from Pyro.errors        import ConnectionClosedError
from common.config      import TIME_ICON_FILE, ID_ICON_FILE, SYNC_ICON_FILE, \
                               UPDATE_ICON_FILE, LOG_ICON_FILE, \
                               PROCESS_ICON_FILE, CALENDAR_ICON_FILE, \
                               IP_CHANGED_ICON_FILE, INCIDENCE_ICON_FILE, \
                               DISCONNECTED_ICON_FILE
from common.utils   import ui_connection_safe

def show_status (centre, where, label, server_ip):

    ''' Mostrar en el control L{where} las incidencias del centro L{centre}.

        @param centre: centro cuyas incidencias se quieren mostrar.
        @type centre: L{list} de L{int} (¿?)
        @param where: control (a su vez con más controles dentro) en el que mostrar la
                      información.
        @type where: L{gtk.Widget}
        @param label: etiqueta que contiene el título del panel.
        @type label: L{gtk.Label}
        @param server_ip: dirección IP del servidor.
        @type server_ip: L{str} '''

    # Para poder modificar las sugerencias:
    toolTips = Tooltips ()
    toolTips.enable ()

    # Se obtienen todas las incidencias:
    def task(myqueue):
        try:
            uia = GraphicalUserInterface.PyroFactory.get_ui_assistant (server_ip)
            inc = uia.get_incidences(centre)
        except ConnectionClosedError:
            print 'ConnectionClosedError (show_status)'
            inc = []

        myqueue.put(inc)
    task = ui_connection_safe(task)
        
    myqueue = Queue()
    thread.start_new_thread(task,(myqueue,))

    while myqueue.empty():
        while gtk.events_pending():
            gtk.main_iteration()
        time.sleep(0.1)

    inc = myqueue.get()

    no_of_inc = len (inc)
    put_title_in_label (label, no_of_inc)
    children = where.get_children ()

    for i in range (len (children)):

        if i + 1 <= no_of_inc:
            # Este hijo debe mostrar una incidencia:
            children [i].show ()

            # Se nombran los nietos relevantes:
            inner = children [i].get_children ()
            title = inner [0]
            iconAndDescription = inner [1].get_children ()
            icon = iconAndDescription [0]
            description = iconAndDescription [1]
            details = inner [2]
            solutions = inner [3].get_children ()

            # 1. Título (nombre de la incidencia):
            tit = inc [i].name

            if tit is None:
                tit = '[Incidencia sin nombre]'
            else:
                tit = tit.strip ()

                if '' == tit:
                    tit = '[Incidencia sin nombre]'

            title.set_markup ('<b>' + str (i + 1) + '. ' + tit + '</b>')

            # 2. Descripción:
            desc = inc [i].description

            if desc is None:
                desc = '[Incidencia sin descripción]'
            else:
                desc = desc.strip ()

                if '' == desc:
                    desc = '[Incidencia sin descripción]'

            description.set_text (desc)

            # 3. Detalles:
            det = inc [i].details

            if det is None:
                details.hide ()
            else:
                det = det.strip ('\n')

                if '' == det.strip ():
                    details.hide ()
                else:
                    details.get_buffer ().set_text (det)
                    details.show ()

            if 'TimeIncidence' == inc [i].type:
                icon.set_from_file (TIME_ICON_FILE)
            elif 'CentreCodeIncidence' == inc [i].type:
                icon.set_from_file (ID_ICON_FILE)
            elif 'SyncIncidence' == inc [i].type:
                icon.set_from_file (SYNC_ICON_FILE)
            elif 'BadLogIncidence' == inc [i].type:
                icon.set_from_file (LOG_ICON_FILE)
            elif 'UpdateIncidence' == inc [i].type:
                icon.set_from_file (UPDATE_ICON_FILE)
            elif 'ParseIncidence' == inc [i].type:
                icon.set_from_file (PROCESS_ICON_FILE)
            elif 'DelayedIncidence' == inc [i].type:
                icon.set_from_file (CALENDAR_ICON_FILE)
            elif 'IpChangedIncidence' == inc [i].type:
                icon.set_from_file (IP_CHANGED_ICON_FILE)
            elif 'ClientPortIncidence' == inc [i].type:
                icon.set_from_file (DISCONNECTED_ICON_FILE)

#             ################ PROVISIONAL ################
#             elif 'TIME' == inc [i].type:
#                 icon.set_from_file (TIME_ICON_FILE)
#             elif 'CENTRECODE' == inc [i].type:
#                 icon.set_from_file (ID_ICON_FILE)
#             elif 'SYNC' == inc [i].type:
#                 icon.set_from_file (SYNC_ICON_FILE)
#             elif 'BADLOG' == inc [i].type:
#                 icon.set_from_file (LOG_ICON_FILE)
#             elif 'UPDATE' == inc [i].type:
#                 icon.set_from_file (UPDATE_ICON_FILE)
#             #############################################

            else:
                icon.set_from_file (INCIDENCE_ICON_FILE)

            # Se muestran los botones correspondientes:
            no_of_inner = len (solutions)
            defined_sols = inc [i].methods

            for j in range (no_of_inner):

                if j + 1 <= len (defined_sols):
                    solutions [j].show ()
                    solutions [j].set_label (defined_sols [j])

                    if 'manual' == defined_sols [j].lower ():
                        toolTips.set_tip (solutions [j], 'Resolver esta incidencia de forma manual (SSH)')
                    else:
                        toolTips.set_tip (solutions [j], None)

                else:
                    solutions [j].hide ()

        else:
            # Ya no hay ms incidencias que mostrar; esconder el hijo:
            children [i].hide ()

        i = i + 1
    showstatus = ui_connection_safe(show_status)

def put_title_in_label (label, no_of_inc = -1):

    ''' Cambiar el título del panel de estado.
        Después de llamar a esta función, la etiqueta muestra un mensaje con
        resaltado, que resume la información que el panel va a contener.

        @param label: el control que muestra el título del panel.
        @type label: L{gtk.Label}
        @param no_of_inc: el nº de incidencias que se van a mostrar, o C{-1}
                          para indicar que no se va a mostrar información
                          en el panel.
        @type no_of_inc: L{int} '''

    if no_of_inc < 0:
        label.set_markup ('<b>Estado</b>')
    elif 0 == no_of_inc:
        label.set_markup ('<b>Estado:</b> sin incidencias')
    elif 1 == no_of_inc:
        label.set_markup ('<b>Estado:</b> una incidencia')
    elif no_of_inc <= 20:
        label.set_markup ('<b>Estado:</b> ' + str (no_of_inc) + ' incidencias')
    else:
        label.set_markup ('<b>Estado:</b> ' + str (no_of_inc) + ' inc. (mostrando 20)')

def solve (GUI, i, j, server_ip, event):

    ''' Resolver una incidencia en un centro.

        @param GUI: interfaz gráfica de usuario en ejecución.
        @type GUI: L{common.gui.gui.GraphicalUserInterface}
        @param i: incidencia.
        @type i: L{int}
        @param j: método de solución de la incidencia.
        @type j: L{int}
        @param server_ip: dirección IP del servidor.
        @type server_ip: L{string}
        @param event: ¿Gum?
        @type event: ¿Gum? '''

    try:
        uia = GraphicalUserInterface.PyroFactory.get_ui_assistant (server_ip)
        inc = uia.get_incidences (GUI.selected)

        uia.solve_incidence (inc [i], j)
    except ConnectionClosedError:
        print 'ConnectionClosedError (solve)'

    except Exception, e:
        print "Excepción controlada:", str(e)

    event.set()

from common.gui.gui import GraphicalUserInterface

# Fin del archivo.

