#!/usr/bin/python
# -*- coding: utf8 -*-

''' Interfaz gráfica de usuario (módulo principal de I{Vaguada}).

    Filtra y valida los argumentos que se le pasen en línea de
    órdenes, y en caso de que éstos sean correctos, instancia un objeto
    L{common.gui.gui.GraphicalUserInterface} del tipo correspondiente
    (I{servidor}, I{diputación} o I{centro}).

    La sintaxis es::
        vaguada.py <ip_servidor> [<código_provincia>[:<código_centro>]]
    donde C{ip_servidor} es una dirección IP válida o un nombre de computador
    válido. Los códigos de provincia y de municipio son números enteros.
    Dependiendo de si se especifica provincia, provincia y municipio,
    o ninguno de los dos, se creará una GUI para la diputación provincial,
    para el centro Guadalinfo, o para el servidor en Sadesi, respectivamente.

    Ejemplos::
        vaguada.py vaguada-server
        vaguada.py localhost 14
        vaguada.py 192.168.1.130 41:606 '''

import pygtk
pygtk.require('2.0')

import gtk

import sys
sys.path.append('./')

from sys            import argv, stderr, exit

from common.config  import PROVINCES, ACT_AS_SERVER
from common.gui.gui import GraphicalUserInterface
try:
    from client.daemon.utils import GuadalinfoId
except:
    pass

if '__main__' == __name__:

    if 2 == len (argv):
        # Primer caso (servidor en Sadesi): se pasa el código "ACT_AS_SERVER":
        server_ip = argv [1].strip ()
        vaguadaInterface = GraphicalUserInterface (ACT_AS_SERVER, server_ip)
        vaguadaInterface.main ()
        exit (0)

    if 3 == len (argv):
        server_ip = argv [1].strip ()
        centre_code = ((argv [2]).strip ()).split (':')

        if 1 == len (centre_code):
            # Segundo caso (diputación provincial): se pasa el código
            # de la provincia, como un entero:
            prov_code = int (centre_code [0])

            if prov_code in [i [0] for i in PROVINCES]:
                vaguadaInterface = GraphicalUserInterface (prov_code, server_ip)
                vaguadaInterface.main ()
                exit (0)

            stderr.write ('%s: ERROR: �el código de provincia %i no es correcto!\n' % \
                          (argv [0], prov_code))
            exit (1)

        if 2 == len (centre_code):
            # Tercer caso (centro Guadalinfo): se pasa una tupla cuyo primer
            # elemento es el código de provincia, y cuyo segundo elemento es el
            # código de municipio (ambos son enteros):

            gid = GuadalinfoId()

            centre_code = (gid['ID_PROVINCIA'], gid['ID_LOCALIDAD'])
            if centre_code == ('0', '0'):
                msg = "El demonio no se está ejecutando.\n"
                msg += "Pruebe a reiniciar el equipo. "
                msg += "Si el problema persiste, contacte" 
                msg += " con el administrador para corregirlo."
                dialog = gtk.MessageDialog(type = gtk.MESSAGE_ERROR, buttons =
                        gtk.BUTTONS_CLOSE, 
                        message_format = msg)
                dialog.run()
                dialog.destroy()
                exit(1)

            if (centre_code [0]).isdigit () and (centre_code [1]).isdigit ():
                prov_code = int (centre_code [0])
                mun_code = int (centre_code [1])

                #if prov_code in [i [0] for i in PROVINCES]:
                if True:
                    vaguadaInterface = GraphicalUserInterface ((prov_code, mun_code), server_ip)
                    vaguadaInterface.main ()
                    exit (0)

                stderr.write ('%s: ERROR: ¡el código de provincia %i no es correcto!\n' % \
                              (argv [0], prov_code))
                exit (1)

        stderr.write ('%s: ERROR: ¡el parámetro "%s" no es válido!\n' % \
                      (argv [0], argv [2]))
        exit (2)

    if 1 == len (argv):
        stderr.write ('%s: ERROR: ¡falta un parámetro!\n' % argv [0])
    elif 4 == len (argv):
        stderr.write ('%s: ERROR: ¡sobra un parámetro!\n' % argv [0])
    else:
        stderr.write ('%s: ERROR: ¡sobran %i parámetros!\n' % (argv [0], len (argv) - 3))

    exit (2)

# Fin del archivo.

